==> forge2d-0.5.0.tar.gz: lib/src/collision/broadphase/default_broadphase_buffer.dart <==
part of forge2d;

/// The broad-phase is used for computing pairs and performing volume queries and ray casts. This
/// broad-phase does not persist pairs. Instead, this reports potentially new pairs. It is up to the
/// client to consume the new pairs and to track subsequent overlap.
class DefaultBroadPhaseBuffer implements TreeCallback, BroadPhase {
  final BroadPhaseStrategy _tree;

  int _proxyCount = 0;

  List<int> _moveBuffer;
  int _moveCapacity = 16;
  int _moveCount = 0;

  List<Pair> _pairBuffer;
  int _pairCapacity = 16;
  int _pairCount = 0;

  int _queryProxyId = BroadPhase.NULL_PROXY;

  DefaultBroadPhaseBuffer(BroadPhaseStrategy strategy) : _tree = strategy {
    _pairBuffer = List<Pair>(_pairCapacity);
    for (int i = 0; i < _pairCapacity; i++) {
      _pairBuffer[i] = Pair();
    }
    _moveBuffer = List<int>.filled(_moveCapacity, 0);
  }

  @override
  int createProxy(final AABB aabb, Object userData) {
    final int proxyId = _tree.createProxy(aabb, userData);
    ++_proxyCount;
    bufferMove(proxyId);
    return proxyId;
  }

  @override
  void destroyProxy(int proxyId) {
    unbufferMove(proxyId);
    --_proxyCount;
    _tree.destroyProxy(proxyId);
  }

  @override
  void moveProxy(int proxyId, final AABB aabb, final Vector2 displacement) {
    final bool buffer = _tree.moveProxy(proxyId, aabb, displacement);
    if (buffer) {
      bufferMove(proxyId);
    }
  }

  @override
  void touchProxy(int proxyId) {
    bufferMove(proxyId);
  }

  @override
  Object getUserData(int proxyId) {
    return _tree.getUserData(proxyId);
  }

  @override
  AABB getFatAABB(int proxyId) {
    return _tree.getFatAABB(proxyId);
  }

  @override
  bool testOverlap(int proxyIdA, int proxyIdB) {
    final AABB a = _tree.getFatAABB(proxyIdA);
    final AABB b = _tree.getFatAABB(proxyIdB);
    if (b.lowerBound.x - a.upperBound.x > 0.0 ||
        b.lowerBound.y - a.upperBound.y > 0.0) {
      return false;
    }

    if (a.lowerBound.x - b.upperBound.x > 0.0 ||
        a.lowerBound.y - b.upperBound.y > 0.0) {
      return false;
    }

    return true;
  }

  @override
  int getProxyCount() {
    return _proxyCount;
  }

  @override
  void drawTree(DebugDraw argDraw) {
    _tree.drawTree(argDraw);
  }

  @override
  void updatePairs(PairCallback callback) {
    // Reset pair buffer
    _pairCount = 0;

    // Perform tree queries for all moving proxies.
    for (int i = 0; i < _moveCount; ++i) {
      _queryProxyId = _moveBuffer[i];
      if (_queryProxyId == BroadPhase.NULL_PROXY) {
        continue;
      }

      // We have to query the tree with the fat AABB so that
      // we don't fail to create a pair that may touch later.
      final AABB fatAABB = _tree.getFatAABB(_queryProxyId);

      // Query tree, create pairs and add them pair buffer.
      _tree.query(this, fatAABB);
    }

    // Reset move buffer
    _moveCount = 0;

    // Sort the pair buffer to expose duplicates.
    buffer_utils.sort(_pairBuffer, 0, _pairCount);

    // Send the pairs back to the client.
    int i = 0;
    while (i < _pairCount) {
      final Pair primaryPair = _pairBuffer[i];
      final Object userDataA = _tree.getUserData(primaryPair.proxyIdA);
      final Object userDataB = _tree.getUserData(primaryPair.proxyIdB);

      callback.addPair(userDataA, userDataB);
      ++i;

      // Skip any duplicate pairs.
      while (i < _pairCount) {
        final Pair pair = _pairBuffer[i];
        if (pair.proxyIdA != primaryPair.proxyIdA ||
            pair.proxyIdB != primaryPair.proxyIdB) {
          break;
        }
        ++i;
      }
    }
  }

  @override
  void query(final TreeCallback callback, final AABB aabb) {
    _tree.query(callback, aabb);
  }

  @override
  void raycast(final TreeRayCastCallback callback, final RayCastInput input) {
    _tree.raycast(callback, input);
  }

  @override
  int getTreeHeight() {
    return _tree.getHeight();
  }

  @override
  int getTreeBalance() {
    return _tree.getMaxBalance();
  }

  @override
  double getTreeQuality() {
    return _tree.getAreaRatio();
  }

  void bufferMove(int proxyId) {
    if (_moveCount == _moveCapacity) {
      final List<int> old = _moveBuffer;
      _moveCapacity *= 2;
      _moveBuffer = List<int>(_moveCapacity);
      buffer_utils.arrayCopy(old, 0, _moveBuffer, 0, old.length);
    }

    _moveBuffer[_moveCount] = proxyId;
    ++_moveCount;
  }

  void unbufferMove(int proxyId) {
    for (int i = 0; i < _moveCount; i++) {
      if (_moveBuffer[i] == proxyId) {
        _moveBuffer[i] = BroadPhase.NULL_PROXY;
      }
    }
  }

  /// This is called from DynamicTree::query when we are gathering pairs.
  @override
  bool treeCallback(int proxyId) {
    // A proxy cannot form a pair with itself.
    if (proxyId == _queryProxyId) {
      return true;
    }

    // Grow the pair buffer as needed.
    if (_pairCount == _pairCapacity) {
      final List<Pair> oldBuffer = _pairBuffer;
      _pairCapacity *= 2;
      _pairBuffer = List<Pair>(_pairCapacity);
      buffer_utils.arrayCopy(oldBuffer, 0, _pairBuffer, 0, oldBuffer.length);
      for (int i = oldBuffer.length; i < _pairCapacity; i++) {
        _pairBuffer[i] = Pair();
      }
    }

    if (proxyId < _queryProxyId) {
      _pairBuffer[_pairCount].proxyIdA = proxyId;
      _pairBuffer[_pairCount].proxyIdB = _queryProxyId;
    } else {
      _pairBuffer[_pairCount].proxyIdA = _queryProxyId;
      _pairBuffer[_pairCount].proxyIdB = proxyId;
    }

    ++_pairCount;
    return true;
  }
}

==> forge2d-0.7.1.tar.gz: lib/src/collision/broadphase/default_broadphase_buffer.dart <==
import 'dart:math';

import '../../../forge2d.dart';
import '../../../src/callbacks/debug_draw.dart';
import '../../../src/callbacks/pair_callback.dart';
import '../../../src/callbacks/tree_callback.dart';
import '../../../src/callbacks/tree_raycast_callback.dart';

/// The broad-phase is used for computing pairs and performing volume queries and ray casts. This
/// broad-phase does not persist pairs. Instead, this reports potentially new pairs. It is up to the
/// client to consume the new pairs and to track subsequent overlap.
class DefaultBroadPhaseBuffer implements TreeCallback, BroadPhase {
  final BroadPhaseStrategy _tree;

  final List<int> _moveBuffer = <int>[];
  final Set<Pair> _pairBuffer = <Pair>{};

  int _queryProxyId = BroadPhase.nullProxy;

  DefaultBroadPhaseBuffer(this._tree);

  @override
  int createProxy(final AABB aabb, Object? userData) {
    final proxyId = _tree.createProxy(aabb, userData);
    _moveBuffer.add(proxyId);
    return proxyId;
  }

  @override
  void destroyProxy(int proxyId) {
    _moveBuffer.remove(proxyId);
    _tree.destroyProxy(proxyId);
  }

  @override
  void moveProxy(int proxyId, final AABB aabb, final Vector2 displacement) {
    final buffer = _tree.moveProxy(proxyId, aabb, displacement);
    if (buffer) {
      _moveBuffer.add(proxyId);
    }
  }

  @override
  void touchProxy(int proxyId) {
    _moveBuffer.add(proxyId);
  }

  @override
  Object? getUserData(int proxyId) {
    return _tree.getUserData(proxyId);
  }

  @override
  AABB getFatAABB(int proxyId) {
    return _tree.getFatAABB(proxyId);
  }

  @override
  bool testOverlap(int proxyIdA, int proxyIdB) {
    final a = _tree.getFatAABB(proxyIdA);
    final b = _tree.getFatAABB(proxyIdB);
    if (b.lowerBound.x - a.upperBound.x > 0.0 ||
        b.lowerBound.y - a.upperBound.y > 0.0) {
      return false;
    }

    if (a.lowerBound.x - b.upperBound.x > 0.0 ||
        a.lowerBound.y - b.upperBound.y > 0.0) {
      return false;
    }

    return true;
  }

  @override
  int getProxyCount() {
    return _moveBuffer.length;
  }

  @override
  void drawTree(DebugDraw argDraw) {
    _tree.drawTree(argDraw);
  }

  @override
  void updatePairs(PairCallback callback) {
    // Reset pair buffer
    _pairBuffer.clear();

    // Perform tree queries for all moving proxies.
    for (final proxyId in _moveBuffer) {
      _queryProxyId = proxyId;
      if (proxyId == BroadPhase.nullProxy) {
        continue;
      }

      // We have to query the tree with the fat AABB so that
      // we don't fail to create a pair that may touch later.
      final fatAABB = _tree.getFatAABB(proxyId);

      // Query tree, create pairs and add them pair buffer.
      _tree.query(this, fatAABB);
    }

    // Reset move buffer
    _moveBuffer.clear();

    // Send the pairs back to the client.
    for (final pair in _pairBuffer) {
      final userDataA = _tree.getUserData(pair.proxyIdA);
      final userDataB = _tree.getUserData(pair.proxyIdB);

      callback.addPair(userDataA, userDataB);
    }
  }

  @override
  void query(final TreeCallback callback, final AABB aabb) {
    _tree.query(callback, aabb);
  }

  @override
  void raycast(final TreeRayCastCallback callback, final RayCastInput input) {
    _tree.raycast(callback, input);
  }

  @override
  int getTreeHeight() {
    return _tree.getHeight();
  }

  @override
  int getTreeBalance() {
    return _tree.getMaxBalance();
  }

  @override
  double getTreeQuality() {
    return _tree.getAreaRatio();
  }

  /// This is called from DynamicTree.query when we are gathering pairs.
  @override
  bool treeCallback(int proxyId) {
    // A proxy cannot form a pair with itself.
    if (proxyId == _queryProxyId) {
      return true;
    }

    _pairBuffer.add(
      Pair(
        min(proxyId, _queryProxyId),
        max(proxyId, _queryProxyId),
      ),
    );
    return true;
  }
}
